<?php class crystal extends baseobject { 
	
	protected static $fields=array(
            'name'=>validator::REQUIRED,
            'prefix'=>validator::REQUIRED,
            'suffix'=>validator::ANY,
            'welldropid'=>array(validator::REQUIRED, validator::INTEGER),
            'dropimageid'=>array(validator::REQUIRED, validator::INTEGER),
            'numberindrop'=>array(validator::REQUIRED, validator::INTEGER),
            'pixelx'=>validator::INTEGER,
            'pixely'=>validator::INTEGER,
            'isdummycoordinate'=>validator::BOOLEAN,
            'isfished'=>validator::BOOLEAN,
	        'spacegroup'=>validator::ANY,
    	    'unitcella'=>validator::FLOAT,
    	    'unitcellb'=>validator::FLOAT,
    	    'unitcellc'=>validator::FLOAT,
    	    'unitcellalpha'=>validator::FLOAT,
    	    'unitcellbeta'=>validator::FLOAT,
    	    'unitcellgamma'=>validator::FLOAT,
	);

	protected static $helpTexts=array(
			'name'=>'A unique name for this crystal',
			'welldropid'=>'The drop where this crystal grew',
			'dropimageid'=>'The image on which the crystal was marked',
			'numberindrop'=>'The scoring system used for this plate',
            'pixelx'=>'Location of the crosshair in pixels from left of image',
            'pixely'=>'Location of the crosshair in pixels from top of image',
            'isdummycoordinate'=>'If true, location was auto-generated, not picked by user',
            'isfished'=>'Whether the crystal has been fished onto a pin for data collection',
	        'spacegroup'=>'The crystal space group',
    	    'unitcella'=>'The crystal\'s unit cell a dimension',
    	    'unitcellb'=>'The crystal\'s unit cell b dimension',
    	    'unitcellc'=>'The crystal\'s unit cell c dimension',
    	    'unitcellalpha'=>'The crystal\'s unit cell &alpha; dimension',
    	    'unitcellbeta'=>'The crystal\'s unit cell &beta; dimension',
    	    'unitcellgamma'=>'The crystal\'s unit cell &gamma; dimension',
	);

	public static $defaultSortOrder='welldropid ASC, numberindrop ASC';
	
	protected static $adminSelect='SELECT SQL_CALC_FOUND_ROWS crystal.*,
                project.name AS projectname, project.isarchived AS isarchived, 
				construct.name AS constructname, protein.name AS proteinname, protein.proteinacronym AS proteinacronym,
                welldrop.name AS welldropname, welldrop.dropnumber AS dropnumber,
                platewell.id AS platewellid, platewell.name AS platewellname, 
                platewell.row AS row, platewell.col AS col,
                plate.id AS plateid, plate.name AS platename,
                dropimage.imagingsessionid AS imagingsessionid
			FROM crystal 
            JOIN dropimage ON dropimage.id=crystal.dropimageid 
            JOIN project ON project.id=crystal.projectid 
            JOIN welldrop ON welldrop.id=crystal.welldropid 
            JOIN platewell ON platewell.id=welldrop.platewellid 
            JOIN plate ON plate.id=platewell.plateid 
			LEFT JOIN construct ON construct.id=welldrop.constructid 
			LEFT JOIN protein ON protein.id=construct.proteinid
            WHERE 1=1
			 ';
	protected static $normalSelect='SELECT SQL_CALC_FOUND_ROWS crystal.*,project.name AS projectname, project.isarchived AS isarchived, 
				construct.name AS constructname, protein.name AS proteinname, protein.proteinacronym AS proteinacronym 
			FROM crystal, project, welldrop
			LEFT JOIN construct ON construct.id=welldrop.constructid 
			LEFT JOIN protein ON protein.id=construct.proteinid
			WHERE project.id=crystal.projectid 
			AND welldrop.id=crystal.welldropid
			 ';
	
	
	public static function getByProperty($key, $value, $request=array()){
		if('plateid'==$key){
			$sqlStatement='SELECT crystal.*, project.name,
				platewell.row AS row, platewell.col AS col, welldrop.dropnumber AS dropnumber, dropimage.imagingsessionid AS imagingsessionid
				FROM crystal,platewell,welldrop,dropimage,project
				WHERE platewell.plateid=:id AND crystal.welldropid=welldrop.id AND welldrop.platewellid=platewell.id 
					AND project.id=platewell.projectid 
					AND dropimage.id=crystal.dropimageid 
					AND dropimage.id=crystal.dropimageid'.
					database::getProjectClause('read')
					.' ORDER BY row, col, dropnumber, crystal.numberindrop
			';
			$params=array(':id'=>$value);
			$sqlStatement.=database::getLimitClause($request);
			
			return database::queryGetAll($sqlStatement, $params);
		}
		return parent::getByProperty($key, $value);
	}
	
	public static function create($request=array()){
		if(!isset($request['welldropid'])){
			return parent::create($request); //will fail validation and throw correct exception
		}
		$welldrop=welldrop::getbyid($request['welldropid']);
		$platewell=platewell::getbyid($welldrop['platewellid']);
		$plate=plate::getById($platewell['plateid']);
		$dropimage=dropimage::getById($request['dropimageid']);
		if(!$dropimage){
			$images=welldrop::gettimecourseimages($request['welldropid']);
			if($images){
				$images=array_reverse($images['rows']);
				
				//first off, is there a visible one?
				foreach ($images as $i){
					if('Visible'==$i['lighttype']){
						$dropimage=$i;
						break;
					}
				}
				//If no visible images, use the first one - SONICC, UV, whatever...
				if(!$dropimage){
					$dropimage=$images[0];
				}
			}
		}
		if(!$dropimage){
		    if(!isset($request['allowcreatedummyinspection'])){
    			throw new BadRequestException('Cannot create crystal because no image was found for this drop');
		    } else {
		        $inspection=imagingsession::createWithDummyImages($plate['id']);
		        $images=welldrop::gettimecourseimages($request['welldropid']);
		        if(empty($images)){
		            throw new BadRequestException('Cannot create crystal because no image was found for this drop. A dummy imaging session could not be created.');
		        }
		        $dropimage=$images['rows'][0];
		    }
		}
		$request['dropimageid']=$dropimage['id'];
		$rowLabels=array('','A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V','Q','X','Y','Z');
		$row=$rowLabels[$platewell['row']];
		$col=$platewell['col'];
		if(strlen($col)<2){ $col='0'.$col; }
		$wellLabel=$row.$col;
		$request['name']=$plate['name'].$wellLabel.'d'.$welldrop['dropnumber'].'c'.$request['numberindrop'];
		if(!empty($welldrop['constructid'])){
			$construct=construct::getbyid($welldrop['constructid']);
			$request['name']=$construct['proteinacronym'].'_'.$request['name'];
		}
		$request['prefix']=$request['name'];
		$created=parent::create($request);
		$diffractionRequest=diffractionrequest::create(array(
		    'crystalid'=>$created['created']['id']
		));
		$created['created']['diffractionrequests']=array();
		$created['created']['diffractionrequests'][]=$diffractionRequest['created'];
		return $created;
	}	
	
	public static function update($id, $request=array()){
		$crystal=static::getbyid($id);
		if(isset($request['prefix']) || isset($request['suffix'])){
			$newPrefix=$crystal['prefix'];
			$newSuffix=$crystal['suffix'];
			if(isset($request['prefix'])){ $newPrefix=trim($request['prefix']); }
			if(isset($request['suffix'])){ $newSuffix=trim($request['suffix']); }
			if(empty($newSuffix)){
				$request['name']=$newPrefix;
			} else {
				$request['name']=$newPrefix.'_'.$newSuffix;
			}
			if(45<strlen($request['name'])){
				throw new BadRequestException('Name was '.strlen($request['name']).' characters long, but cannot be more than 45 characters (limit imposed by synchrotron ISPyB system).');
			}
		}
		return parent::update($id, $request);
	}
	
	public static function delete($id){
	    $crystal=static::getById($id);
	    if(!$crystal){ throw new NotFoundException('No crystal with ID '.$id); }
	    if(1==$crystal['isfished']){ throw new ForbiddenException('Crystal cannot be deleted because it has been fished.'); }
        return parent::delete($id);	    
	}
	
	public static function getdiffractionrequests($id,$request=array()){
	    $ret=diffractionrequest::getByProperty('crystalid', $id, $request);
	    if(!$ret){
	        //Crystal existed before we did things this way.Create an empty shipping request
	        $created=diffractionrequest::create(array(
	            'crystalid'=>$id,
	            'diffractiontype'=>'OSC' //Doing this in code, not as a database default, in case we ned to handle non-ISPyB synchrotrons in future
	        ));
	        $rows=array();
	        $rows[]=$created['created'];
	        $ret=array('total'=>1, 'rows'=>$rows);
	        $GLOBALS['forceCommit']=true;
	    }
	    return $ret;
	}
}

?>