<?php 

//define class autoloader
spl_autoload_register(function($className){
    $paths=array(
        '../classes/',
        '../classes/core/',
        '../classes/core/exception/',
        '../classes/core/authentication/',
        '../classes/core/interface/',
        '../classes/model/',
        './XraySourceInterface/',
    );
    foreach($paths as $path){
        if(file_exists($path.$className.'.class.php')){
            include_once($path.$className.'.class.php');
        }
    }
});
    
//connect to the database
database::connect();

//Get a session
$firstUser=database::queryGetOne('SELECT id from user ORDER BY id ASC LIMIT 1');
session::init(new PhpSession());
session::set('isAdmin',true);
session::set('userId',$firstUser['id']);

//Determine the current code version
$icebearCodeVersion="1.0"; //default
$codeVersionFile=@file_get_contents('../conf/codeversion');
if(!empty($codeVersionFile)){
    $icebearCodeVersion=($codeVersionFile);
}

//Determine the current database version
$icebearDbVersion=config::get('core_icebearversion');

echo "\nIcebear DB version: ".$icebearDbVersion;
echo "\nIcebear code version: ".$icebearCodeVersion;
if($icebearCodeVersion==$icebearDbVersion){
    echo "\nVersions match, nothing to do.";
    exit;
}

$upgraded=false;

while($icebearCodeVersion!=$icebearDbVersion){
    $dirs=new DirectoryIterator('.');
    foreach($dirs as $dir){
        $ranAnUpgrade=false;
        if(!$dir->isDir() || $dir->isDot()){ continue; }
        $name=$dir->getFilename();
        if(0===strpos($name, 'v'.str_replace('.', '_', $icebearDbVersion).'-to-v')){
            echo "\nFound upgrade directory ".$name."";
            if(!file_exists(rtrim($name,'/').'/upgrade.php')){
                echo "\nFound upgrade directory but no upgrade.php inside it, exiting.";
                exit(1);
            }
            echo "\nRunning upgrade...";
            $parts=explode('-to-v', $name);
            $versionAfterUpgrade=str_replace('_', '.', $parts[1] );
            try {
                database::begin();
                include(rtrim($name,'/').'/upgrade.php');
                database::commit();
                config::set('core_icebearversion',$versionAfterUpgrade);
                $icebearDbVersion=$versionAfterUpgrade;
                echo"\n...Finished upgrade ".$name.".";
                $ranAnUpgrade=false;
            } catch(Exception $e){
                database::abort();
                echo "\nException thrown during upgrade:";
                echo "\n\n".$e->getMessage()."\n\nLine: File:";
                $trace=$e->getTrace();
                
                foreach ($trace as $t){
                    echo "\n".str_pad($t['line'],5, ' ', STR_PAD_LEFT).' '.$t['file'];
                }

                echo "\nUpgrade script aborted with error.";
                echo "\nIcebear DB version: ".$icebearDbVersion;
                echo "\nIcebear code version: ".$icebearCodeVersion;
                
                exit;
            }
        }
        if($icebearCodeVersion==$icebearDbVersion){
            echo "\n\nCode and database versions match.\n";   
            break;
        }
    }
    if(!$ranAnUpgrade){ break; }
}

if($icebearCodeVersion==$icebearDbVersion){
    echo "\n\nCode and database versions now match.\n";
} else {
    echo "\n\nWARNING:\n";
    echo "\nScript completed without errors, but ";
    echo "\nCode and database versions DO NOT MATCH.";
    echo "\nIf database version is later than code";
    echo "\nversion, you will need to update the code.";
}


echo "\n\nDatabase upgrade script completed.";
exit;


/* SUPPORTING FUNCTIONS */


/**
 * Adds the column to the table, if it doesn't exist.
 * Returns true if created, false if already existed
 */
function addColumnIfNotExists($table, $column, $definition){
    if(!preg_match('/^[A-Za-z0-9_-]+$/', $table)){ throw new BadRequestException('Bad table name '.htmlentities($table).' got to addColumnIfNotExists'); }
    if(!preg_match('/^[A-Za-z0-9_-]+$/', $column)){ throw new BadRequestException('Bad column name '.htmlentities($column).' got to addColumnIfNotExists'); }
    if(!preg_match('/^[\(\)\s\.\,\'\"A-Za-z0-9_-]+$/', $definition)){ throw new BadRequestException('Bad definition '.htmlentities($definition).' got to addColumnIfNotExists'); }
    if(!empty(database::queryGetAll('SHOW COLUMNS in '.$table.' LIKE "'.$column.'"'))){
        echo "\nTable $table already has column $column";
        return false;
    }
    database::query(" ALTER TABLE $table ADD $column $definition ");
    echo "\nAdded column $column to $table";
    return true;
}
?>